/*
 * Copyright 2016-2017 Univention GmbH
 *
 * http://www.univention.de/
 *
 * All rights reserved.
 *
 * The source code of this program is made available
 * under the terms of the GNU Affero General Public License version 3
 * (GNU AGPL V3) as published by the Free Software Foundation.
 *
 * Binary versions of this program provided by Univention to you as
 * well as other copyrighted, protected or trademarked materials like
 * Logos, graphics, fonts, specific documentations and configurations,
 * cryptographic keys etc. are subject to a license agreement between
 * you and Univention and not subject to the GNU AGPL V3.
 *
 * In the case you use this program under the terms of the GNU AGPL V3,
 * the program is provided in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public
 * License with the Debian GNU/Linux or Univention distribution in file
 * /usr/share/common-licenses/AGPL-3; if not, see
 * <http://www.gnu.org/licenses/>.
 */
/*global define*/

define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/on",
	"dojo/topic",
	"umc/dialog",
	"umc/tools",
	"umc/widgets/Page",
	"umc/widgets/Form",
	"umc/widgets/Module",
	"umc/widgets/TextBox",
	"umc/widgets/ComboBox",
	"umc/widgets/PasswordBox",
	"umc/widgets/Text",
	"umc/widgets/Wizard",
	"umc/i18n!umc/modules/zimbra"
], function(declare, lang, array, on, topic, dialog, tools, Page, Form, Module, TextBox, ComboBox, PasswordBox, Text, Wizard, _) {

	var ZimbraWizard = declare('umc.modules.zimbra.ZimbraWizard', [Wizard], {
		umcpCommand: null,
		closeModule: null,
		autoValidate: true,
		autoFocus: true,
		'class': 'umcCard',

		constructor: function() {
			this.inherited(arguments);

			lang.mixin(this, {
				pages: [{
					name: 'start',
					headerText: _('Welcome to the Zimbra configuration'),
					helpText: _('Welcome to the Zimbra setup wizard. A few steps are needed to complete the configuration process.'),
					widgets: [{
						type: Text,
						name: 'select_hostname',
						content: _('To connect Zimbra with UCS, a Zimbra Collaboration Suite with a configured administator account and one mail domain is required.') + ' ' +
							_('The Zimbra Collaboration Suite software can be found at <a href="http://www.zimbra.com/downloads/">http://www.zimbra.com/downloads/</a>.') + ' ' +
							_('For experimental use Zimbra Collaboration Suite can also be installed in a docker container, explained in this <a href="https://wiki.zimbra.com/wiki/Deploy_Zimbra_Collaboration_using_docker">documentation</a>.') + '<br/>' +
							_('Please enter the hostname of the Zimbra Collaboration Suite.'),
					}, {
						type: TextBox,
						name: 'hostname',
						label: _('Hostname (FQHN) of Zimbra Colaboration Server'),
						value: ''
					}]
				}, {
					name: 'certificate',
					headerText: _('Setup zimbra certificate'),
					helpText: _(''),
					widgets: [{
						type: Text,
						name: 'ssl',
						content: _('In order to setup the SSL certificate please login via ssh to the Zimbra Collaboration Suite and execute the following command: <br />sudo -i -u zimbra sh -c "zmlocalconfig -e ssl_allow_untrusted_certs=true && zmmailboxdctl restart"')
					}]
				}, {
					name: 'credentials',
					headerText: _('Setup zimbra administrator user account'),
					helpText: _('In this step a zimbra administrator account is needed...'),
					widgets: [{
						type: Text,
						name: 'select_credentials',
						content: _('Please enter the credentials of an zimbra administrator user.')

					}, {
						type: TextBox,
						name: 'username',
						label: _('Username')
					}, {
						type: PasswordBox,
						name: 'password',
						label: _('Password')
					}]
				}, {
					name: 'maildomain',
					headerText: _('Select mail domain'),
					helpText: _('Please select a mail domain from the UCS environment. Zimbra Collaboration Suite will be configured to accept mails for that domain.'),
						// Ja, was eigentlich? Das sollte hier noch beschrieben werden. Wozu muss man die Maildomain auswählen? Der Text oben ist ein Vorschlag.
					widgets: [{
						type: ComboBox,
						name: 'maildomain',
						label: _('Mail domain'),
						staticValues: []
					}]
				}, {
					name: 'success',
					headerText: _('Configuration complete'),
					helpText: _('The configuration of Zimbra Collaboration Server has been completed successfully. New users can now be created. Please note the tab "Zimbra" in the user menu and define the appropriate user role and attributes.')
						// Muss bei den Nutzern noch etwas mit angegeben werden? Kann ein Link auf das Benutzermodul für den Text "New users can now be created" gesetzt werden? Das wäre großartig.
				}, {
					name: 'initalized',
					headerText: _('Zimbra Colaboration Server is configured'),
					helpText: _('The configuration of Zimbra Colaboration Server has already been completed.')
				}, {
					name: 'error',
					headerText: _('An error occurred'),
					helpText: '',
					widgets: [{
						type: Text,
						name: 'error',
						content: _('An error occurred. It might help to restart the wizard.')
					}]
				}]
			});
		},

		hasNext: function(pageName) {
			if (~array.indexOf(['initalized', 'success', 'error'], pageName)) {
				return false;
			}
			return this.inherited(arguments);
		},

		hasPrevious: function(pageName) {
			if (~array.indexOf(['initalized', 'success', 'error'], pageName)) {
				return false;
			}
			return this.inherited(arguments);
		},

		canCancel: function(pageName) {
			if (~array.indexOf(['success', 'error'], pageName)) {
				return false;
			}
			return this.inherited(arguments);
		},

		next: function(pageName) {
			var nextPage = this.inherited(arguments);
			if (nextPage === 'start') {
				return this.standbyDuring(this.umcpCommand('zimbra/query').then(lang.hitch(this, function(data) {
					if (data.initialized) {
						return 'initalized';
					}
					return nextPage;
				}), lang.hitch(this, 'closeModule')));
			} else if (pageName === 'credentials') {
				return this.standbyDuring(this.umcpCommand('zimbra/get_mail_domains', this.getValues()).then(lang.hitch(this, function(data) {
					this.getWidget('maildomain', 'maildomain').set('staticValues', data.result);
					return nextPage;
				})));
			} else if (pageName === 'maildomain') {
				return this.standbyDuring(this.umcpCommand('zimbra/configure', this.getValues()).then(lang.hitch(this, function(data) {
					return nextPage;
				})));
			}
			return nextPage;
		}

	});

	return declare("umc.modules.zimbra", [Module], {
		_wizard: null,
		unique: true,

		postMixInProperties: function() {
			this.inherited(arguments);
			this._wizard = new ZimbraWizard({
				umcpCommand: lang.hitch(this, 'umcpCommand'),
				closeModule: lang.hitch(this, 'closeModule')
			});
			this._wizard.on('finished', lang.hitch(this, 'closeModule'));
			this._wizard.on('cancel', lang.hitch(this, 'closeModule'));
		},

		buildRendering: function() {
			this.inherited(arguments);
			this.addChild(this._wizard);
		}
	});
});
